/*
 * Decompress.cpp
 * 
 * Written by Matt Pauker
 *
 * 5/11/96
 */

#include "Decompress.h"
#include "StatusWindow.h"

extern char **environ;

void DoError(const char *errorMsg)
// Displays the string in errorMsg in a BAlert
{
	BAlert *myAlert;
	
	myAlert = new BAlert("", errorMsg, "Cancel", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
	myAlert->Go();
}

void CopyExtraFields(BRecord *sourceRecord, BRecord **destRecord)
// Copies all "extra" fields from sourceRecord to destRecord
{
	long x, size;
	char *fieldName;
	void *data;

	x = 0;
	
	while (sourceRecord->GetExtraInfo(x, &fieldName, &size) == B_NO_ERROR)
	{
		data = sourceRecord->FindExtra(fieldName, &size);
		(*destRecord)->SetExtra(fieldName, data, size);
		x++;
	}
}


bool IsGzipFile(record_ref fileRef)
// Checks whether the file represented by fileRef is a gzip file or not
{
	BFile file;
	char *data = new char[2];
	bool returnVal;
	
	file.SetRef(fileRef);
	file.Open(B_READ_ONLY);
	if (file.Size() > 2)
	{
		file.Read(data, 2);
		file.Close();
		
		returnVal = ((data[1] == ''));	// The second char of a gzip file is always an 
										// It's possible it might mistake a non-gzip file, but
										// this should be enough
		delete[] data;
		
		return returnVal;
	}
	else
	{
		file.Close();
		return FALSE;
	}
}

void CreateTempFile(BFile *tempFile, BDirectory *tempDir)
// Creates a temporary file in the /tmp directory and returns its record_ref
{
	record_ref dirRef;
	
	get_ref_for_path("/tmp", &dirRef);
	tempDir->SetRef(dirRef);
	tempDir->Create("CompressTemp", tempFile, NULL, NULL, NULL);
}


#pragma export on
void process_refs(record_ref dirRef, BMessage *message, void *data);
#pragma export reset

void process_refs(record_ref dirRef, BMessage *message, void *data)
{
	ulong type;
	long numRefs;
	long result;
	
	message->GetInfo("refs", &type, &numRefs);
	
	if (numRefs == 0) /* Nothing selected */
		DoError("You must select a file to decompress.");
	else
	{
		BDirectory dir;

		dir.SetRef(dirRef);
		
		if (dir.Error() != B_NO_ERROR)
			DoError("You must select a file within a folder.");
		else
		{
			record_ref ref;
			char *path = new char[1000];
			long x;
			
			// Set the current directory to dir, so that gzip can find the file
			dir.GetPath(path, 1000);
			chdir(path);
	
			for (x = 0; x < numRefs; x++)
			{
				ref = message->FindRef("refs", x);
				if (does_ref_conform(ref, "File"))
				{
					BFile file;
					char *fileName = new char[B_FILE_NAME_LENGTH];
					char *newFileName = new char[B_FILE_NAME_LENGTH];
					
					file.SetRef(ref);
					file.GetName(fileName);

					if (IsGzipFile(ref))
					{
						char **passargv;
						thread_id gzipThread;
						thread_info info;
						BRecord *record;
						ulong *type, *app;
						long length;
						BFile newFile, tempFile;
						BDirectory tempDir;
						
						// If the file had been gzip'd with the Compress add-on, it should contain
						// some extra fields with the original type and app signatures.  If so,
						// retrieve them.
						record = file.Record();
						type = (ulong *)record->FindExtra("compressType", &length);
						app  = (ulong *)record->FindExtra("compressApp", &length);
						
						// The file's extra fields need to be stored, so create a temp file in which to
						// store them.
						CreateTempFile(&tempFile, &tempDir);
						record = tempFile.Record();
						CopyExtraFields(file.Record(), &record);

						// Set up the arguments to send to gzip
						passargv = (char **)malloc((4) * (sizeof(char *)));
					
						passargv[0] = "/boot/bin/gzip";
						passargv[1] = "-d";
						passargv[2] = fileName;
						passargv[3] = 0;
				
						// Try to load gzip
						gzipThread = load_posix(passargv[0], passargv, environ);
						if (gzipThread > 0)
						{
							resume_thread(gzipThread);	// Start it
							
							snooze(400000.0);	// Wait a bit
							
							// If it's still running (File must have been large), show a status
							// window							
							if (get_thread_info(gzipThread, &info) == B_NO_ERROR)
							{
								StatusWindow *statWindow;

								statWindow = new StatusWindow();
								statWindow->Show();
								wait_for_thread(gzipThread, &result);
								statWindow->Close();
							}
						
							// If the file had a type and app stored, set them in the new file
							if ((type != NULL) && (app != NULL))
							{
								// Get the new file
								strncpy(newFileName, fileName, strlen(fileName) - 3);
								newFileName[strlen(newFileName) + 1] = 0;
								dir.GetFile(newFileName, &newFile);
								
								// Set its type and app signatures
								newFile.SetTypeAndApp(*type, *app);
							
								// Remove the type and app fields so they aren't copied to the new
								// file
								tempFile.Record()->RemoveExtra("compressType");
								tempFile.Record()->RemoveExtra("compressApp");	

								// Copy the extra fields from the tempFile to the new file
								record = newFile.Record();								
								CopyExtraFields(tempFile.Record(), &record);
								record->Commit();
							}
						}
						else
							DoError("Sorry, I couldn't find the gzip tool.");
						
						free(passargv);
						tempDir.Remove(&tempFile);
					}
					else
						DoError("That file doesn't seem to have been compressed with gzip.");

					delete[] fileName;
					delete[] newFileName;
				}
				else
					DoError("You can only decompress files.");
			}
		}
	}
}
